"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Container = /** @class */ (function () {
    function Container() {
        this.resolvables = new Map();
        this.values = new Map();
    }
    Container.prototype.register = function (type, implementedBy, lifetimeScope, constructorParameters) {
        var resolvable = {
            implementation: implementedBy,
            lifetimeScope: lifetimeScope,
            constructorParameters: constructorParameters,
        };
        this.resolvables.set(type, resolvable);
        if (lifetimeScope === "permanent") {
            this.values.set(type, this.resolveTransient(resolvable));
        }
    };
    Container.prototype.registerValue = function (key, value) {
        var resolvable = { lifetimeScope: "permanent" };
        this.resolvables.set(key, resolvable);
        this.values.set(key, value);
    };
    Container.prototype.resolve = function (key) {
        var resolvedResolvable = this.resolvables.get(key);
        if (resolvedResolvable === undefined) {
            throw new Error("Cannot resolve type " + this.getKeyName(key) + ".");
        }
        switch (resolvedResolvable.lifetimeScope) {
            case "transient":
                return this.resolveTransient(resolvedResolvable);
            case "permanent":
                return this.resolvePermanent(key);
            default:
                throw new Error("Unknown lifetime scope.");
        }
    };
    Container.prototype.getResolver = function () {
        return this.resolve.bind(this);
    };
    Container.prototype.resolveTransient = function (resolvable) {
        var _a;
        return new ((_a = (resolvable.implementation)).bind.apply(_a, [void 0].concat((resolvable.constructorParameters || []))))();
    };
    Container.prototype.resolvePermanent = function (type) {
        var resolvedInstance = this.values.get(type);
        if (resolvedInstance === undefined) {
            throw new Error("Cannot resolve type " + this.getKeyName(type) + ".");
        }
        return resolvedInstance;
    };
    Container.prototype.getKeyName = function (key) {
        if (typeof (key) === "string") {
            return key;
        }
        return key.name;
    };
    return Container;
}());
exports.Container = Container;
exports.globalContainer = new Container();
exports.globalResolve = exports.globalContainer.getResolver();
